@@ -25,6 +25,9 @@ import itertools
 import functools
 import inspect
 from abc import abstractmethod
+import logging
+
+logger = logging.getLogger(__name__)
 
 class YPException(Exception):
     '''Exception thrown by the engine.'''
@@ -33,6 +36,10 @@ class YPException(Exception):
 class IUnifiable(object):
     '''Base interface for all term types that can be unified.'''
 
+    @abstractmethod
+    def get_value(self):
+        '''gets the instantiated value of this object.'''
+
     @abstractmethod
     def to_python(self):
         '''converts the object's Prolog value to Python.'''
@@ -50,6 +57,8 @@ class Atom(IUnifiable):
     def name(self):
         '''return the atom's name.'''
         return self._name
+    def get_value(self):
+        return self
     def to_python(self):
         if self._name == '[]':
             return []
@@ -116,6 +125,9 @@ class Functor(IUnifiable):
         """
         self._name = name
         self._args = args
+    def get_value(self):
+        valargs = [ get_value(a) for a in self._args ]
+        return Functor(self._name, valargs)
     def to_python(self):
         if self._name == ".":
             # listpair
@@ -179,8 +191,8 @@ def chain_functions(func1, func2):
 
 def get_value(v):
     """Return the value of a Prolog term. Constants and functors will return themselves,
-    variables will be expanded if bound. Does not recursively expand variables."""
-    if isinstance(v, Variable):
+    variables will be expanded if bound. Recursively expands variables."""
+    if isinstance(v, IUnifiable):
         return v.get_value()
     return v
 
@@ -287,9 +299,29 @@ class YP(object):
         if False:
                 yield False
 
+    def findall(self, template, goal, bag):
+        '''findall/3 returns values according to template into bag, that satisfy goal.'''
+        # assumes goal is instantiated
+        q = self.query(goal._name,goal._args)
+        results = self.makelist([ get_value(template) for r in q ])
+        for y in unify(bag, results):
+            yield False
+
+    def call(self, goal):
+        '''call/1 calls goal.'''
+        yield from self.query(goal._name, goal._args)
+
+    def once(self, goal):
+        '''once/1 calls goal only once.'''
+        q = self.call(goal)
+        yield next(q)
+
     def _set_builtin_predicates(self):
         self.register_function('=', builtin_eq)
         self.register_function('\\=', self.builtin_neq)
+        self.register_function('findall', self.findall)
+        self.register_function('call', self.call)
+        self.register_function('once', self.once)
 
     def clear(self):
         """clears all defined atoms, variables, facts and rules."""
@@ -341,8 +373,8 @@ class YP(object):
         If overwrite is True, it will overwrite existing function definitions. Otherwise,
         function definitions will be combined. Functions with the same name but a different
         number of arguments will be overwritten or combined. This could cause runtime errors.
-        Note: since yldprolog version 1.1.0, generated function names include the arity,
-        making this problem less likely.
+        Note: since yldprolog version 1.1.0, the compiler generates function names that include
+        the arity, making this problem less likely.
         """
         new_context = self.eval_context.copy()
         code = compile(s, fn, 'exec')
